/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ECHARTBASE_H
#define ECHARTBASE_H

#include <ewidget.h>
#include "echarts.h"

#include "echartvalues.h"

class EAction;

/**
	* @defgroup Charts
	*
	* This module contains all classes, which are related to the charts library of libengsas.
	*
	* @image html example-echarts-screenshot.png
	*/


/**
	* @brief Base class for ECharts.
	* @ingroup Charts
	*
	* To add values, call addDataSet(). How the values are treated
	* or how many data sets you can add, depensd on the specific implementation.
	*
	* Your chart should be drawn in your implementation of drawChart().
	* This function is called by paintEvent() and also by exportChart()
	* to paint the chart on an image. Therefore, you should only use
	* the given view port size within this function.
	*
	* To add an export action to your context menu or something else,
	* just call exportAction() to create the action.
	*
	* If you must draw text within your subclass, use labelFont or may be titleFont.
	*/
class E_CHARTS_EXPORT EChartBase : public EWidget
{
	Q_OBJECT
	Q_PROPERTY(QString chartTitle READ chartTitle WRITE setChartTitle)
	Q_PROPERTY(int numberPrecision READ numberPrecision WRITE setNumberPrecision)
	Q_PROPERTY(char numberFormat READ numberFormat WRITE setNumberFormat)
	Q_PROPERTY(QFont titleFont READ titleFont WRITE setTitleFont)
	Q_PROPERTY(QFont labelFont READ labelFont WRITE setLabelFont)
	public:
		EChartBase(QWidget *parent = NULL);

		void setChartTitle(QString chartTitle);
		inline QString chartTitle() const { return myChartTitle; }
		void setNumberPrecision(int precision);
		inline int numberPrecision() const { return myNumberPrecision; }
		void setNumberFormat(char format);
		inline char numberFormat() const { return myNumberFormat; }

		/**
			* @brief Adds a new line to the graph.
			*
			* If x-ticks and y-ticks are not numeric, x- and y-values
			* can only contain values from x-ticks and y-ticks.
			*
			* If @p color is empty, a default order is used.
			*/
		void addDataSet(EChartValues dataSet, const QString label, QColor color = QColor());
		bool removeDataSet(const QString label);

		void setTitleFont(QFont font);
		QFont titleFont() const;
		void setLabelFont(QFont font);
		QFont labelFont() const;

		EAction *exportAction() const;

	public slots:
		void exportChart();
		void exportChart(QPainter *painter, QRect boundingRect);
		void clear();

	protected:
		void paintEvent(QPaintEvent *event);
		/**
			* @brief Function to draw the chart.
			*
			* It is called by the overladed paintEvent() function
			* and also used to export the chart into an image.
			*
			* @note Make sure to restore the painter afterwards!
			*/
		virtual void drawChart(QPainter *painter, QRect viewPort) = 0;

		int marginHorizontal, marginVertical;
		QRect viewPort;
		QStringList colorNames;
		QList<void (*)(QPainter*, QPoint)> markers;
		QMap<QString, EChartValues> dataSets;
		QMap<QString, QColor> dataSetsColors;
		QMap<QString, void (*)(QPainter *, QPoint)> dataSetsMarkers;

	protected slots:
		/**
			* @brief This slot is called, if the context menu of this chart should be shown.
			*
			* You may reimplement this function, if you need another menu
			* than the default one.
			*/
		virtual void contextMenu(QPoint pos) const;

	private:
		/**
			* @brief Draws the default legend on the right side of the chart.
			*
			* Is called by paintEvent() after drawChart().
			*/
		void legend(QPointF origin, QPainter *painter);
		/**
			* @brief Calculates the view port, paints it and returns it's size
			*
			* Is called by paintEvent() before drawChart().
			*/
		QRect createViewPort(QPainter *painter, QRect boundingRect);

		char myNumberFormat;
		int myNumberPrecision;
		QString myChartTitle;
		QFont myTitleFont, myLabelFont;
		int legendMarkersMargin;
};

#endif // ECHARTBASE_H
