/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ECHARTLINE_H
#define ECHARTLINE_H

#include "echarts.h"
#include "echartbase.h"

#include "echartvalues.h"

#include <QMap>
#include <QColor>
#include <QVariant>

/**
	* @brief A EWidget to display line charts
	* @ingroup Charts
	*
	* The title of the whole chart can be set by setTitle().
	* Afterwards set the title of the x- (setXTitle()) and y-axis (setYTitle()).
	* Now configure the x- and y-axis with setXTicks() and setYTicks().
	* The chart is configured now and you can add lines with addDataSet().
	* Make sure, that the values are added in the order of x- and y-ticks.
	* If x-ticks and y-ticks are not numeric, x- and y-values can only contain
	* exactly the values from x-ticks and y-ticks.
	*
	* If chart is numeric (x-ticks and y-ticks contains only numeric values)
	* the precision and the format of the displayed numbers can be configured with
	* setNumberPrecision() and setNumberFormat(). Per default the format is 'f' and
	* the precision 2 (e. g. 123.45).
	*
	* @todo calculate the linesDrawn only if line is added, the x- or y-ticks
	*       are changed or the widget size is changed.
	* @todo Provide context menu to export chart values and chart as image
	*/
class E_CHARTS_EXPORT EChartLine : public EChartBase
{
	Q_OBJECT
	Q_PROPERTY(QString xTitle READ xTitle WRITE setXTitle)
	Q_PROPERTY(QString yTitle READ yTitle WRITE setYTitle)
	Q_PROPERTY(int xTicksAngle READ xTicksAngle WRITE setXTicksAngle)
	public:
		EChartLine(QWidget *parent = NULL);

		void setYTitle(QString yTitle);
		inline QString yTitle() const { return myYTitle; }
		void setXTitle(QString xTitle);
		inline QString xTitle() const { return myXTitle; }
		void setXTicksAngle(int angle);
		/**
		 * @brief Returns the angle of the x-ticks labels.
		 *
		 * 0degree is horizontal and than it turns clockwise (e. g. vertical from top to bottom
		 * is equal to 90degree). Default is 0 and maximum is 360degree.
		 *
		 * @see setXTicksAngle()
		 */
		int xTicksAngle() const;

		void setXTicks(const QList<QVariant> xTicks);
		void setYTicks(const QList<QVariant> yTicks);

	protected:
		void drawChart(QPainter *painter, QRect viewPort);
		/**
			* @brief Used to display the measurements
			*/
		void mouseMoveEvent(QMouseEvent * event);

	private:
		float linearInterpolation(const float dy, const float x1, const float x0, const float y0, const float x) const;

		QString myXTitle, myYTitle;
		QList<QVariant> xTicks, yTicks;
		bool xNumeric, yNumeric;
		/**
			* The points currently used to draw the lines
			*/
		QMap<QString, QList<QPointF> > linesDrawn;
        QPoint origin, currentPosition;
		qreal currentXValue, currentYValue;
		int myXTicksAngle;
};

#endif // ECHARTLINE_H
