/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EABSTRACTITEMMODEL_H
#define EABSTRACTITEMMODEL_H

#include "eglobal.h"
#include <QAbstractItemModel>

/**
	* @brief Replacement of QAbstractItemModel to provide convenience signals for message handling
	*
	* @note Since rowsInserted() and rowsRemoved() are a private signals of QAbstractItemModel, they
	*       are not useable with subclasses of EAbstractItemModel. Therefore EAbstractItemModel provides
	*       the public signals eRowsInserted() and eRowsRemoved() as replacements, which are also emitted,
	*       if rowsInserted() btw. rowsRemoved() are emitted.
	*       This should not effect subclasses of QAbstractItemView, because QAbstractItemViews are
	*       friends of QAbstractItemModel.
	*/
class E_CORE_EXPORT EAbstractItemModel : public QAbstractItemModel
{
	Q_OBJECT
	public:
		EAbstractItemModel(QObject *parent = 0);
		inline virtual ~EAbstractItemModel(){};
		
		static void connectByPass(QObject *parent, EAbstractItemModel *child);
		static void connectByPass(EAbstractItemModel *parent, QObject *child);

	protected slots:
		/**
			* @see error() if a child emits this signal, this class emits error()
			* (it passes the childs signal upwards)
			* @see EWidget::errorByPass()
			* @see EDialog::errorByPass()
			*/
		virtual inline void errorByPass(const QString &message){ emit error(message); };
		/**
			* @see warning() passes the emitted signals from it's child forward
			* @see EWidget::warningByPass()
			* @see EDialog::warningByPass()
			*/
		virtual inline void warningByPass(const QString &message){ emit warning(message); };
		/**
			* @see information() passes the emitted signals from it's child forward
			* @see EWidget::informationByPass()
			* @see EDialog::informationByPass()
			*/
		virtual inline void informationByPass(const QString &message){ emit information(message); };
		
	private slots:
		void rowsInsertedByPass(const QModelIndex &parent, int start, int end);
		void rowsRemovedByPass(const QModelIndex &parent, int start, int end);
		
	signals:
		void error(QString msg);
		void warning(QString msg);
		void information(QString msg);
		/**
		  * @brief Replacement of private signal rowsInserted().
		  */
		void eRowsInserted(const QModelIndex &parent, int start, int end);
		/**
			* @brief Replacement of private signal rowsRemoved().
			*/
		void eRowsRemoved(const QModelIndex &parent, int start, int end);
};

#endif // EABSTRACTITEMMODEL_H
