/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EMAINWINDOW_H
#define EMAINWINDOW_H

#include "eglobal.h"
#include <QMainWindow>

#include "ehelpcentersetup.h"

class EWidget;
class EAction;
class ESettingsModel;
class EHelpCenter;
class EDependencyModel;
class ESetupWizardObject;
class EAboutModel;

#include <QUrl>

class QMenu;
class QStatusBar;
class QMenuBar;
class QSignalMapper;

/**
	* @brief Base class for mainwindows. It provides several useful functions by default.
	*
	* Especially the handling of EWidget's is quite well. It also set's the QApplication::organizationName() to EngSaS
	* and allows better standard conform window title handling. It also provides file opening (fileOpen()),
	* file save (fileSave() and fileSaveAs())and recently used files functionalities.
	*
	* You may subclass some of the protected slots, but if you use EWidget's, this shouldn't
	* be needed, because all specials must be done in your EWidget's.
	*
	* @note It hides the QMenu others if it is empty
	* @todo Implement auto hide of QMenu, if childs are removed (e.g. via deleting the child somewhere else)
	* @todo Save and restore size
	* @see EWidget
	*/
class E_CORE_EXPORT EMainWindow : public QMainWindow
{
	Q_OBJECT
	public:
		EMainWindow(QWidget* parent = 0, Qt::WindowFlags flags = 0);
		virtual ~EMainWindow();
		void setWindowTitle ( const QString &title );

	protected slots:
		virtual void fileNew();
		virtual void fileOpen();
		virtual void fileOpen(QString absoluteFilePath);
		virtual void fileSave();
		virtual void fileSaveAs();
		/**
			* @brief Function called by fileSave() and fileSaveAs() to actually do save data.
			*
			* This function just calls the EWidget::save() function from the current
			* central widget, if it is an EWidget.
			*
			* @note The file is not opened or closed, because opening it
			*       with QIODevice::WriteOnly, truncates it and the EWidget
			*       has no chance to extract already present data.
			*/
		virtual void fileDoSave(QString file);
		virtual void filePrint();
		virtual void fileExit();
		virtual void editCut();
		virtual void editCopy();
		virtual void editPaste();
		virtual void editUndo();
		virtual void editRedo();
		/**
			* @brief Implement this function to call showSetupWizard() properly.
			*
			* This slot is connected to the setup wizard action in edit menu.
			* Normally you setup a list of ESetupWizardObjects in this function
			* and call showSetupWizard() with this list and true as second argument,
			* to inform the user, if the wizard is not needed.
			* @code
			*   ESetupWizardObjects objects;
			*   // Add your objects here with objects << yourPointerToAnObject;
			*   showSetupWizard(objects, true);
			* @endcode
			* @see showSetupWizard()
			* @see ESetupWizard
			* @see ESetupWizardObject
			* @see ESetupWizardWidget
			*/
		virtual inline void editSetupWizard(){ }
		/**
			* @brief Connect to the Settings action in Edit menu.
			*
			* Subclass this function, to create a ESettingsModel and call showSettings(ESettingsModel*)
			* afterwards, to present the content to the user with help of the standard ESettingsDialog.
			* @see showSettings()
			*/
		virtual inline void editSettings(){ showSettings(0); }
		virtual void viewReload();
		/**
			* @brief Calls helpHelp(QString) with empty string to open standard help.
			*
			* @see helpHelp(QString)
			*/
		void helpHelp();
		/**
			* @brief Starts the helpcenter and tries to load the given @p file.
			*
			* This function is also called by the slot showHelp(QString).
			* @see showHelp(QString)
			* @see setupHelpCenter()
			*/
		virtual void helpHelp(QString file);
		/**
			* @brief Calls showDependencies() with a NULL-Pointer.
			*
			* Implement this function to call showDependencies() with your
			* configured EDependencyModel.
			*/
		virtual inline void helpDependencies(){ showDependencies(0); }
		/**
			* @brief Calls showAbout() with empty values, should therefore be implemented at your subclass.
			*
			* Implement this function in your subclass to call showAbout() with proper values.
			* @see showAbout()
			*/
		virtual void helpAbout();
		/**
			* @brief Reload your settings here.
			*
			* Please emit doReloadSettings() or call this base implementation to
			* broadcast the reload event to all childs.
			*/
		virtual inline void reloadSettings(){ emit doReloadSettings(); }

		/**
			* used to disable or enable default actions
			*/
		void enableActions(EngSaS::AvailableActions enabledActions);
		/**
			* @see EWidget::connectESlots()
			* @see EObject::connectESlots()
			*/
		void connectESlots(QObject *object);

		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls error().
			*/
		inline void errorByPass(const QString &message){ error(message); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls warning().
			*/
		inline void warningByPass(const QString &message){ warning(message); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls information()
			*/
		inline void informationByPass(const QString &message){ information(message); }
		/**
			* @brief Forwards the reloading of enabled actions signal.
			*
			* These functions are wrappers to provide convenient API to EWidget and EObject.
			*/
		inline void reloadEnableActionsByPass(){ reloadEnableActions(); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls addMenu()
			*/
		inline void addMenuByPass(QMenu* newMenu, const QString &parent, QString before = QString()){ addMenu(newMenu, parent, before); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls addAction()
			*/
		inline void addActionByPass(QAction* newAction, const QString &parent, QString before = QString()){ addAction(newAction, parent, before); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls addToolBar()
			*/
		inline void addToolBarByPass(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea){ addToolBar(newToolBar, area); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls addNewDockWidget()
			*/
		inline void addNewDockWidgetByPass(QDockWidget *widget, const Qt::DockWidgetArea &area){ addNewDockWidget(widget, area); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls setCentralWidget()
			*/
		inline void setCentralWidgetByPass(QWidget *centralWidget){ setCentralWidget(centralWidget); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls showSettings()
			*/
		inline void showSettingsByPass(){ showSettings(); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			* Calls showHelp()
			*/
		inline void showHelpByPass(const QString &page){ showHelp(page); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			*/
		inline void showStatusWidgetByPass(QWidget* widget){ showStatusWidget(widget); }
		/**
			* these functions are wrappers to provide convenient API to EWidget and EObject.
			* For example EObject::connectByPass() can be called with help of these slots.
			*/
		inline void reloadSettingsRequestByPass(){ reloadSettings(); }
		/**
			* @brief Clears the list of recently used files.
			*/
		void clearRecentlyUsed();

	protected:
		/**
			* @brief Uses ESetupWizard to guide the user through the setup process.
			*
			* It shows the ESetupWizard containing @p objects, if needed. If not and
			* @p msgOnNotNeeded is set to true, it popsup a message box with the information,
			* that everything is correctly configured. With @p logo and @p applicationName,
			* you can setup the logo and application name used inside the wizard. If nothing
			* is set, the default of ESetupWizard is used (QApplication::windowIcon() and
			* QApplication::applicationName()).
			* To call this function with proper
			* arguments, implement editSetupWizard().
			* @see ESetupWizard
			* @see editSetupWizard()
			*/
		void showSetupWizard(EngSaS::SetupWizardObjects objects,
												 bool msgOnNotNeeded = false,
												 QIcon logo = QIcon(),
												 QString applicationName =  QString());
		/**
			* @brief Uses ESettingsDialog to present the settings given in the @p model.
			*
			* It adds the settings widget from ELibrary also.
			*/
		void showSettings(ESettingsModel *model);
		/**
			* @brief Show the standard EDepenenciesDialog.
			*
			* @see helpDependencies()
			*/
		void showDependencies(EDependencyModel *model);
		/**
			* @brief Shows the standard EAboutDialog as about dialog.
			*
			* It adds the EABoutPersons from ELibrary also.
			*
			* @note Normally @p url should be relative to engsas.de. EAboutDialog::setApplicationUrl()
			*   made it absolute with the localized engsas url.
			*/
		void showAbout(QPixmap logo, QString name, QString version, QUrl url, EAboutModel *model);
		/**
			* @brief Call this function in your subclass to setup the EHelpCenter.
			*
			* @see EHelpCenter
			* @see EHelpCenterSetup
			*/
		void setupHelpCenter(EHelpCenterSetup setup);
		/**
			* Stores the geomtry and state.
			*/
		void closeEvent(QCloseEvent *event);
		/**
			* Calls restoreGeometry() and restoreState(), therefore
			* it should be called after all widgets are loaded.
			* It uses objectName() to identify itself, so the objectname
			* should be setted also before calling this function.
			*/
		void restoreWindowState();
		
		/**
			* @see EWidget::error()
			* @see EObject::error()
			* @see warning()
			* @see information()
			*/
		void error(const QString &message);
		/**
			* @see EWidget::warning()
			* @see EObject::warning()
			* @see error()
			* @see information()
			*/
		void warning(const QString &message);
		/**
			* @see EWidget::information()
			* @see EObject::information()
			* @see error()
			* @see warning()
			*/
		void information(const QString &message);
		/**
			* @brief Indicates, that a reload of the enabled actions is needed.
			*/
		void reloadEnableActions();
		/**
			* If @p parent can not be found, it is added to toplevel
			* menu "Other". This menu is enabled if needed
			* With help of @p before, the action, which should be the one before @p newAction
			* can be specified (QMenu::menuAction()).
			* @todo Sort order (Especially in fileMenu insert before Exit).
			*   Maybe it should be possible to provide an "after" menu or action
			* @note Be aware, that the @p parent menu becomes the parent. Therfore
			*   you are responsible to delete the menu again, if your widget is deleted
			* @see EWidget::addMenu()
			* @see EObject::addMenu()
			* @todo Find out why QMenu::setParent() messes up the action relations and strip out the workaround
			*/
		void addMenu(QMenu* newMenu, const QString &parent, QString before = QString());
		/**
			* If no @p parent is given or the given parent can not be found,
			* the action is added to the menu "Other", which is enabled if needed.
			* With help of @p before, the action, which should be the one before @p newAction
			* can be specified.
			* @note Be aware, that the @p parent menu becomes the parent. Therfore
			*   you are responsible to delete the menu again, if your widget is deleted
			* @see EWidget::addAction()
			* @see EObject::addAction()
			*/
		void addAction(QAction* newAction, const QString &parent, QString before = QString());
		/**
			* @see EWidget::addToolBar()
			* @see EObject::addToolBar()
			*/
		void addToolBar(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea);
		/**
			* @see EWidget::addNewDockWidget()
			* @see EObject::addNewDockWidget()
			*/
		void addNewDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area);
		/**
			* @see EWidget::setCentralWidget()
			* @see EObject::setCentralWidget()
			*/
		void setCentralWidget(QWidget *centralWidget);
		/**
			* @see EWidget::showSettings()
			* @see EObject::showSettings()
			*/
		inline void showSettings(){ editSettings(); }
		/**
			* @see EWidget::showHelp()
			* @see EObject::showHelp()
			*/
		void showHelp(const QString &page);
		/**
			* @see EWidget::showStatusWidget()
			* @see EObject::showStatusWidget()
			*/
		void showStatusWidget(QWidget* widget);
		
		/**
			* tries to cast the central widget and returns a pointer the resulting object
			*/
		EWidget *getCentralWidget();
		void controleMenuStati();
		void controleMenuStati(QMenu *menu);
		void addToRecentlyUsed(QString absoluteFilePath);

		QMenuBar *menubar;
		QStatusBar *statusbar;
		QMenu *fileMenu, *fileRecentlyUsedMenu, *editMenu, *viewMenu;
		QMenu *otherMenu, *helpMenu;
		EAction *fileNewAction;
		EAction *fileOpenAction;
		EAction *fileSaveAction;
		EAction *fileSaveAsAction;
		EAction *filePrintAction;
		EAction *fileExitAction;
		EAction *editUndoAction;
		EAction *editRedoAction;
		EAction *editCutAction;
		EAction *editCopyAction;
		EAction *editPasteAction;
		EAction *editSetupWizardAction;
		EAction *editSettingsAction;
		EAction *viewReloadAction;
		EAction *helpHelpAction;
		EAction *helpDependenciesAction;
		EAction *helpAboutAction;
		EHelpCenter *helpCenter;
		EHelpCenterSetup helpCenterSetup;
		QList<QAction*> recentlyUsedActions;
		QAction *recentylUsedSeperator;
		QSignalMapper *recentlyUsedMapper;

	private:
		/**
			* @internal
			* @return The QAction or QAction related to QMenu, which has object name @p name and is child of @p parent
			*/
		QAction *findBefore(QObject *parent, QString name);
		/**
			* @internal
			* @brief Setups the UI Elements like menus, statusbar, ...)
			*/
		void setupUi();
		/**
			* @brief Adds the needed actions to fileRecentlyUsedMenu.
			*/
		void loadRecentlyUsed();

	signals:
		void doReloadSettings();
};

#endif // EMAINWINDOW_H
