/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EOBJECT_H
#define EOBJECT_H

#include "eglobal.h"
#include <QObject>

class ESettingsWidget;

#include <QStringList>
#include <QPointer>

class QFile;
class QMenu;
class QAction;
class QToolBar;
class QDockWidget;

/**
	* @brief Mostly the same as EWidget but for QObject as base class.
	* @see Ewidget
	* @see EDialog
	*/
class E_CORE_EXPORT EObject : public QObject
{
	Q_OBJECT
	public:
		inline EObject ( QObject *parent = 0 ) : QObject(parent){}
		virtual ~EObject();
		/**
			* Connects all standard signals.
			* @see EWidget::connectByPass()
			* @see EDialog::connectByPass()
			*/
		void connectByPass(QObject *object){ EObject::connectByPass(this, object); }
		/**
			* Connects all standard signals between @p parent and @p child.
			* @code
			*   // this can be also an EWidget or EObject or EDialog
			*   EWidget *child = new EWidget(this);
			*   // connect all signals and slots as they must be connected
			*   EObject::connectByPass(this, child);
			* @endcode
			* Normally you would call connectByPass(QObject*) and not this function directly
			*/
		static void connectByPass ( QObject *parent, QObject *child );

	public slots:
		/**
			* @brief Used to reload the settings.
			*
			* This function is connected to doReloadSettings() and should be used
			* to actually do the reload of the settings. To indicate, that the settings
			* should be reloaded, emit reloadSettingsRequest().
			* Use @p error to describe the error for the user. And don't forget to
			* call the base implementation or emit doReloadSettings() yourself.
			*
			* @see doReloadSettings()
			* @see reloadSettingsRequest()
			* @see EWidget::reloadSettings()
			* @see EDialog::reloadSettings()
			*/
		virtual inline void reloadSettings(){ emit doReloadSettings(); }

	protected slots:
		/**
			* @see error() if a child emits this signal, this class emits error()
			* (it passes the childs signal upwards)
			* @see EWidget::errorByPass()
			* @see EDialog::errorByPass()
			*/
		virtual inline void errorByPass(const QString &message){ emit error(message); }
		/**
			* @see warning() passes the emitted signals from it's child forward
			* @see EWidget::warningByPass()
			* @see EDialog::warningByPass()
			*/
		virtual inline void warningByPass(const QString &message){ emit warning(message); }
		/**
			* @see information() passes the emitted signals from it's child forward
			* @see EWidget::informationByPass()
			* @see EDialog::informationByPass()
			*/
		virtual inline void informationByPass(const QString &message){ emit information(message); }
		/**
			* Passes the reloadEnableActions() signals from it's childs forward.
			*/
		virtual inline void reloadEnableActionsByPass(){ emit reloadEnableActions(); }
		/**
			* @see addMenu() passes the emitted signals from it's child forward
			* @see EWidget::addMenuByPass()
			* @see EDialog::addMenuByPass()
			*/
		virtual inline void addMenuByPass(QMenu* newMenu, const QString &parent, QString before = QString()){ emit addMenu(newMenu, parent, before); }
		/**
			* @see addAction() passes the emitted signals from it's child forward
			* @see EWidget::addActionByPass()
			* @see EDialog::addActionByPass()
			*/
		virtual inline void addActionByPass(QAction* newAction, const QString &parent, QString before = QString()){ emit addAction(newAction, parent, before); }
		/**
			* @see addToolBar() passes the emitted signals from it's child forward
			* @see EWidget::addToolBarByPass()
			* @see EDialog::addToolBarByPass()
			*/
		virtual inline void addToolBarByPass(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea){ emit addToolBar(newToolBar, area); }
		/**
			* @see addNewDockWidget() passes the emitted signals from it's child forward
			* @see EWidget::addNewDockWidgetByPass()
			* @see EDialog::addNewDockWidgetByPass()
			*/
		virtual inline void addNewDockWidgetByPass(QDockWidget *widget, const Qt::DockWidgetArea &area){ emit addNewDockWidget(widget, area); }
		/**
			* @see setCentralWidget() passes the emitted signals from it's child forward
			* @see EWidget::setCentralWidgetbyPass()
			* @see EDialog::setCentralWidgetbyPass()
			*/
		virtual inline void setCentralWidgetByPass(QWidget *centralWidget){ emit setCentralWidget(centralWidget); }
		/**
			* @see showSettings() passes the emitted signals from it's child forward
			* @see EWidget::showSettingsByPass()
			* @see EDialog::showSettingsByPass()
			*/
		virtual inline void showSettingsByPass(){ emit showSettings(); }
		/**
			* @see showHelp() passes the emitted signals from it's child forward
			* @see EWidget::showHelpByPass()
			* @see EDialog::showHelpByPass()
			*/
		virtual inline void showHelpByPass(const QString &page){ emit showHelp(page); }
		/**
			* @see showStatusWidget() passes the emitted signals from it's child forward
			* @see EWidget::showStatusWidgetbyPass()
			* @see EDialog::showStatusWidgetbyPass()
			*/
		virtual inline void showStatusWidgetByPass(QWidget* widget){ emit showStatusWidget(widget); }
		/**
			* @see reloadSettingsRequest() passes the emitted signals from it's child forward.
			* @see reloadSettings()
			* @see reloadSettingsByPass()
			* @see doReloadSettings()
			*/
		virtual inline void reloadSettingsRequestByPass(){ emit reloadSettingsRequest(); }

	protected:
		/**
			* @brief Adds an dock widget via addNewDockWidget() and adds @p widget to the list of managed dock widgets.
			*
			* If @p actionLabel (and also actionParent) is set, an EAction is created and added to parent @p actionParent to show and
			* hide the dock widget. This action is also managed and therefore deleted along with the dock widget itself.
			*
			* The benefit is, that all the dock widgets on this list are deleted automatically, if the widget
			* it self is deleted.
			*/
		void addManagedDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area,
															QString actionLabel = QString(), QString actionParent = QString());

	private:
		QList<QPointer<QDockWidget> > dockWidgets;
		QList<QPointer<QAction> > dockActions;
		
	signals:
		/**
			* The signals error(), warning() and information() are used to handle
			* user information. error() should be used in critical situations.
			* warning() is intendend to inform the user about not critical,
			* but important facts. information() are only to inform the user,
			* but they have no bad results.
			* To let the main application decide, how they are displayed,
			* only signals are emitted.
			* This signal is used to notify about an error with @p message
			* @see warning()
			* @see information()
			* @see errorByPass()
			* @see EWidget::error()
			* @see EDialog::error()
			*/
		void error(const QString &message);
		/**
			* Used to notify about an warning with @p message.
			* @see error()
			* @see information()
			* @see warningByPass()
			* @see EWidget::warning()
			* @see EDialog::warning()
			*/
		void warning(const QString &message);
		/**
			* Used to notify about an information with @p message.
			* @see error()
			* @see warning()
			* @see informationByPass()
			* @see EWidget::information()
			* @see EDialog::information()
			*/
		void information(const QString &message);
		/**
			* @brief Request reloading of enabled actions in EMainWindow.
			*/
		void reloadEnableActions();
		/**
			* Allows to add menu @p newMenu to the main window.
			* It is added to an existing menu with QObject::objectName() @p parent
			* before the action or menu with object name @p before. If @p before is empty,
			* it is appended at the bottom.
			* @see addAction() to add an action
			* @see addToolBar()
			* @see addMenuByPass()
			* @see EWidget::addMenu()
			* @see EDialog::addMenu()
			*/
		void addMenu(QMenu* newMenu, const QString &parent, QString before = QString());
		/**
			* Adds the action @p newAction to the main window.
			* It is added to an existing menu with QObject::objectName() @p parent
			* before the action or menu with object name @p before. If @p before is empty,
			* it is appended at the bottom.
			* @see addMenu()
			* @see addToolBar()
			* @see addActionByPass()
			* @see EWidget::addAction()
			* @see EDialog::addAction()
			*/
		void addAction(QAction* newAction, const QString &parent, QString before = QString());
		/**
			* Used to add the toolbar @p newToolBar to the main window.
			* @see addMenu()
			* @see addAction()
			* @see addToolBarByPass()
			* @see EWidget::addToolBar()
			* @see EDialog::addToolBar()
			*/
		void addToolBar(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea);
		/**
			* Used to add the dock widget @p widget to the main window at the given @p area.
			* The main window automatically adds an checkabe show and hide action to the view menu
			* with the title of the @p widget.
			* @see setCentralWidget()
			* @see addNewDockWidgetByPass()
			* @see EWidget::addNewDockWidget
			* @see EDialog::addNewDockWidget
			*/
		void addNewDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area);
		/**
			* Sets @p centralWidget as the new central widget.
			* @attention old central widget is destroyed
			* @see addNewDockWidget()
			* @see setCentralWidget()
			* @see EWidget::setCentralWidget()
			* @see EDialog::setCentralWidget()
			*/
		void setCentralWidget(QWidget *centralWidget);
		/**
			* request to show the settings dialog.
			* @see showSettingsByPass()
			* @see EWidget::showSettings()
			* @see EDialog::showSettings()
			*/
		void showSettings();
		/**
			* help should be shown and @p page should be opened.
			* @see showHelpByPass()
			* @see EWidget::showHelp()
			* @see EDialog::showHelp()
			*/
		void showHelp(const QString &page);
		/**
			* Adds @p widget to the status bar as a permanent widget.
			* @see showStatusWidgetByPass()
			* @see EWidget::showStatusWidget()
			* @see EDialog::showStatusWidget()
			*/
		void showStatusWidget(QWidget* widget);
		/**
			* @brief Indicates, that a reload of the settings is needed.
			*
			* Therefore this signal is used to bring the reload message upwards to the main window
			* or whatever.
			* @see reloadSettingsRequestByPass()
			* @see doReloadSettings()
			* @see reloadSettings()
			* @see EWidget::reloadSettingsRequest()
			* @see EDialog::reloadSettingsRequest()
			*/
		void reloadSettingsRequest();
		/**
			* @brief Emitted to bring the reload request back to the farest child.
			* @see reloadSettings()
			* @see reloadSettingsRequest()
			* @see EWidget::doReloadSettings()
			* @see EDialog::doReloadSettings()
			*/
		void doReloadSettings();
};

#endif // EOBJECT_H
