/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EQUERYTHREAD_H
#define EQUERYTHREAD_H

#include "eglobal.h"
#include <QThread>

class EQueryWorker;

#include <QSqlQuery>
#include <QSqlRecord>
#include <QVariant>

/**
	*@brief Allows to execute SQL-Queries asynchronous.
	* @ingroup SQL
	*
	* Should be stopped right that:
	* @code
	* EQueryThread.cancelQuery(); // to stop running query
	* thread.quit(); // to stop eventloop
	* thread.wait(1000); // to give the thread max 1s to be stopped
	* @endcode
	*/
class E_CORE_EXPORT EQueryThread : public QThread
{
	Q_OBJECT
  public:
    EQueryThread( QObject* parent = 0 );
    ~EQueryThread();
		
    void run();
		bool isStarted() const;
		bool waitForStart();
		
	public slots:
		/**
			* @brief Adds a query to the queue.
			*
			* Prepares an QSqlQuery with query @p queryString. Afterwards it
			* binds all values from @p bindValues. This values must be ordered (Placeholder ? not :fieldName).
			* To allow the identification of the result by the @p client, use @p requestId. The related
			* emit of the result() signal, contains the unchagend @p requestId and also the pointer to @p client.
			* To display an error message if the query failed, set the @p errorMsg to a meaningful message.
			*/
		void execQuery(QObject *client, QVariant requestId,
									 QString queryString, QString errorMsg,
									 QList<QVariant> bindValues = QList<QVariant>());
		void cancelQuery();
		void dbSetup(QString dbname, QString username, QString password, QString host, QString driver);
		
	private slots:
		/**
			* @brief Internally used to by pass the fetched results.
			*/
		void resultByPass(QObject *client, QVariant requestId, int querySize, QSqlRecord record);
		void setRangeByPass(int min, int max);
		void setValueByPass(int value);
		void finishedByPass(QObject* client, QVariant requestId, bool status);
		void setStarted();
		
	private:
		/**
			* this object run's completely at the other thread
			* therefore the communication is done through the event loop 
			* with help of signals and slots
			*/
		EQueryWorker *worker;
		bool finishedStartup;
		
	signals:
		/**
			* used to configure database at QueryWorker
			*/
		void configureDB(QString dbname, QString username, QString password, QString host, QString driver);
		/**
			* @brief Adds a new query to the queue of the query worker (Internally used).
			*
			* @note This signal is only used to communicate with the QueryWorker.
			*
			* The query itself is stored in @p queryString. To use placeholders, add them in
			* @p bindValues. To allow the processing of the results at the connected objects correctly,
			* the @p requestId is appended unchanged to emitted result() signal for this query.
			* To display an error message if the query failed, set the @p errorMsg to a meaningful message.
			*/
		void addToQueue(QObject *client, QVariant requestId, QString queryString, QString errorMsg, QList<QVariant> bindValues);
		void cancel();
		/**
			* @brief Is emitted, if a result for a queued query is fetched.
			*
			* The @p client and @p requestId are unchanged and should be used by the caller
			* to identify it's queries.
			*/
		void result(QObject *client, QVariant requestId, int querySize, QSqlRecord record);
		void setRange(int min, int max);
		void setValue(int value);
		/**
			* @brief Emitted to indicate the result status of a query.
			*
			* To identify the query, the @p client and @p requestId is also
			* given.
			*/
		void finished(QObject *client, QVariant requestId, bool status);
		void allConnected();
};

#endif // EQUERYTHREAD_H
