/*
    Copyright (C) 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ESPREADSHEETHANDLER_H
#define ESPREADSHEETHANDLER_H

#include "eglobal.h"
#include "eobject.h"

class ESpreadsheet;
class ESpreadsheetFile;
class ESpreadsheetProvider;
class ESettingsModelItem;

#include <QMultiMap>
#include <QStringList>

/**
	* @defgroup Spreadsheets
	*
	* This module contains all classes, which provide functionalities to the topic spreadsheet im- and export.
	*
	*/

/**
	* @brief Class to read and write files containing spreadsheet.
	* @ingroup Spreadsheets
	*
	* Use this class to read and write to spreadsheet files. It uses
	* ESpreadsheetProvider, to handle specific formats (like csv, ods, xls, ...).
	* Which provider it uses, is determined by the file suffix, given to loadFile() and saveFile().
	* Which suffixes are supported, can be listed with filters().
	*
	* To read a spreadsheet file, use loadFile(QString) or loadFile(QFile) to load the file.
	* Afterwards you can call spreadsheets() to get a list of spreadsheet within the loaded file.
	* Each spreadsheet is represented by one ESpreadsheet object. Use those objects to read and or
	* modify the data within a spreadsheet.
	*
	* To create a new spreadsheet file, call createFile(QString) or createFile(QFile). Than you can
	* add new spreadsheets to the file by calling appendSpreadsheet(). Than modify the ESpreadsheet object
	* to add data to the spreadsheet.
	*
	* If you want to handle a new file type, you must implement a new ESpreadsheetProvider.
	*
	*/
class E_CORE_EXPORT ESpreadsheetHandler : public EObject
{
	Q_OBJECT
	public:
		ESpreadsheetHandler(QObject *parent = NULL);
		~ESpreadsheetHandler();
		
		/**
			* @brief A list with all ESpreadsheetProvider
			*/
		static QList<ESpreadsheetProvider*> provider();
		/**
			* @brief The settings page for the ESpreadsheetHandler module.
			*
			* Does also contain the page for the available ESpreadsheetProvider.
			*/
		static ESettingsModelItem* settings();
		/**
			* @brief A list of supported file filters, combineable by ";;".
			*/
		QStringList filters() const;
		/**
			* @brief Returns true, if a provider is able to handle files with suffix @p fileSuffix.
			*/
		bool handles(QString fileSuffix) const;
		/**
			* @brief Creates a file containing the spreadsheets.
			*
			* It uses a provider, which supports the current file suffix.
			*
			* @see create(QFile*)
			* @see load(QString)
			* @see load(QFile*)
			* @see writeData()
			* @see close()
			*/
		bool create(QString file);
		bool create(QFile *file);
		/**
			* @brief Loads the file containing the spreadsheets.
			*
			* @see create(QString)
			* @see create(QFile*)
			* @see load(QFile*)
			* @see writeData()
			* @see close()
			*/
		bool load(QString file);
		bool load(QFile *file);
		/**
			* @brief Call this function after modifying the ESpreadsheet to write the data to the filesystem.
			*
			* @see load(Qstring)
			* @see load(QFile*)
			* @see create(QString)
			* @see create(QFile*)
			* @see close()
			*/
		bool writeData();
		/**
			* @brief Closes the opened file.
			*
			* To save changed data before closing the file, call writeData().
			*
			* @see load(Qstring)
			* @see load(QFile*)
			* @see create(QString)
			* @see create(QFile*)
			* @see close()
			*/
		void close();
		ESpreadsheet* appendSpreadsheet(QString name);
		EngSaS::Spreadsheets spreadsheets() const;

	private:
		/**
			* @brief Tries to load a provider with help of the suffix from @p file.
			*
			* It sets ESpreadsheetHandler::provider to NULL, therefore if it returns false,
			* ESpreadsheetHandler::provider is also NULL.
			*/
		bool loadProvider(QString file);

		ESpreadsheetProvider *currentProvider;
		QMultiMap<QString, ESpreadsheetProvider* > mapFilterToProvider;

};

#endif // ESPREADSHEETHANDLER_H

/**
	* @example espreadsheethandler.cpp
	*
	* @brief This example demonstrates the usage of ESpreadsheetHandler to load and display spreadsheets.
	*
	* The example has one main window and allows to load and save spreadsheet files (like Excel, CSV, ...)
	* with help of ESpreadsheetHandler and ESpreadsheetProvider implementations.
	*
	* @image html example-espreadsheethandler-screenshot.png
	*
	* The main window is a class with name MainWindow, defined in file mainwindow.h as follows:
	* @snippet examples/espreadsheethandler/mainwindow.h MainWindow
	*
	* The implementation is done within mainwindow.cpp. Within the constructor, the UI is created and the
	* ESpreadsheetHandler is loaded. At this point, it is also checked, that at least one ESpreadsheetProvider
	* is available, to make sure that some spreadsheet formats could be handled.
	*
	* Since the Open action in the menu bar is connected to the slot QMainWindow::close(), only the MainWindow::open()
	* and MainWindow::save() slots are implemented within this class.
	* @snippet examples/espreadsheethandler/mainwindow.cpp Constructor
	*
	* If the Open action in the menubar is clicked, the slot MainWindow::open() is executed. After checking for
	* an available handler, it asks the user for a file to open. The open dialog allows to choose all files, which
	* can be handled by the available providers. To achieve this, the ESpreadsheetHandler::filters() are forwarded
	* to the dialog.
	*
	* The file is opened with ESpreadsheetHandler::load(QString) after checking for empty files. If the file was loaded
	* successfully, the number of contained sheets is determined with ESpreadsheetHandler::spreadsheets() and checked.
	*
	* At least the QStandardItemModel of the view is updated with new data of the first sheet.
	* @snippet examples/espreadsheethandler/mainwindow.cpp Open
	*
	* To save changed data, it calls ESpreadsheetHandler::writeData().
	* @snippet examples/espreadsheethandler/mainwindow.cpp Save
	*
	* espreadsheethandler.cpp contains the main() function to execute QApplication and load the MainWindow.
	*/
