/*
		Copyright (C) 2010 - 2015 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EXMLMODEL_H
#define EXMLMODEL_H

#include "eglobal.h"
#include "eabstractitemmodel.h"

class EXmlItem;
class EXmlParserItem;

#include <QModelIndex>
#include <QDomDocument>
#include <QFile>
#include <QStringList>
#include <QUrl>

/**
	* @brief ExmlModel is a base class to handle XML files very easily
	*
	* To get an working XMl Model, you only have to subclass it and create an Constructor.
	* Inside the constructor, append you header strings to headerList and register your
	* subclassed EXmlItem's with help of registerParser().
	*
	* To hide the rootItem, use QAbstractItemView::setRootIndex() and set it to the index
	* of the rootItem.
	*
	* If your model needs to be prepared before it could be saved, implement prepareForSaving()
	* in your subclass.
	*
	* To copy the complete model, create a new empty model and set a copy of the original root item
	* as the new root item.
	* @code
	* EXmlModel *newModel = new EXmlModel();
	* newModel->setRootItem(oldModel->rootItem()->copy());
	* @endcode
	*
	* @note All tagnames used here are turned into lowercase.
	* @see EXmlItem
	* @see EXmlStandardRootItem
	* @see headerList
	*/
class E_CORE_EXPORT EXmlModel : public EAbstractItemModel
{
	Q_OBJECT
	public:
		/**
			* @p validationUrl can point to a local file via file:// (e.g. file:///tmp/xmlschema.xsd)
			* or to a url (e.g. http://www.example.org/example.dtd)
			* @see EDependencyModel as an example implementation
			*/
		EXmlModel(QUrl validationUrl, QObject *parent = 0);
		virtual ~EXmlModel();
		
		// own api
		/**
			* @brief Loads the XML structure below and including @p element.
			* 
			* Returns true if everything went fine (e.g. every tag could be parsed)
			*/
		bool load(QDomElement element);
		/**
			* @brief Overladed of load(QDomElement)
			*/
		bool load(QDomDocument document);
		/**
			* @brief Overladed of load(QDomDocument)
			*/
		bool load(QFile *file);
		/**
			* @brief Overladed of load(QFile*)
			*/
		bool load(QString file);
		/**
			* @brief Overloaded of load(QDomDocument).
			* 
			* Constructs a QDomDocument out of the QByteArray.
			*/
		bool load(QByteArray data);
		/**
			* The model is stored into @p useElem.
			* But be aware, that @p useElem will contain the rootItem of this model.
			* Therefore useElem should be an empty element (but not a null element)
			*/
		bool save(QDomElement &useElem);
		/**
			* overladed of save(QDomElement)
			*/
		bool save(QFile *file);
		/**
			* overladed of save(QFile*)
			*/
		bool save(QString file);
		/**
			* @brief The model contained in a QDomDocument.
			* 
			* The document is in utf-8 encoding. If an error occurs,
			* an empty QDomDocument is returned.
			*/
		QDomDocument save();
		/**
			* checks the actual structure validation against the scheme at validationUrl
			* @return true if the structure is valid.
			*/
		bool isValidXML();
		EXmlItem *itemFromIndex(QModelIndex index) const;
		QModelIndex indexFromItem(EXmlItem *item) const;
		/**
			* @brief Removes and deletes the item with index @p index.
			*/
		bool removeItem(QModelIndex index);
		/**
			* @brief Removes and deletes the item @p item.
			*
			* Conveniance function for removeItem(QModelIndex())
			*/
		bool removeItem(EXmlItem *item);

		/**
			* @brief Replaces or sets the current root item to @p item.
			*
			* If an existing item is replaced, the signals modelAboutToBeReset() and
			* modelReset() are emitted. Otherwise the signals layoutAboutToBeChanged()
			* and layoutChanged() are emited.
			*/
		virtual void setRootItem(EXmlItem* item);
		EXmlItem *rootItem() const;

		/**
		 * @brief Returns true, if the model has been modified since the last save.
		 *
		 * If the model is modified for the first time after a save() or load(),
		 * isModified() is set to true and also the signal modified(true) is emitted.
		 * Saving the model with save() resets the modified state to false (emits also modified(false)
		 * and the process starts over again.
		 *
		 * To detect those changes, EXmlItem::changed() is monitored by the model and also
		 * row inserts and removes are observed.
		 */
		bool isModified() const;


		// standard api
		QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
		bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
		int columnCount(const QModelIndex& parent = QModelIndex()) const;
		int rowCount(const QModelIndex& parent = QModelIndex()) const;
		QModelIndex parent(const QModelIndex& child) const;
		QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
		QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
		bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex());

	signals:
		void modified(bool status);
		
	protected:
		/**
			* call this function to register your subclass of EXmlItem.
			* Your subclass should contain a public static function to create
			* the needed function pointer @p createCallback.
			* This Function pointer is called during loading the structure, to
			* create instances of your subclass and parse the given XML structure
			* @returns true if the @p createCallback could be registered
			* to handle tags with name @p tagName.
			* @returns false, if not. This can be the case, if @p tagName is empty, or
			* @p createCallback is NULL
			*
			* The created EXmlParserItem by this function automatically provides
			* the pointer to this EXmlModel, when calling the @p createCallback.
			*
			* @see EXmlItem
			* @see EDependencyItem as an example implementation
			*/
		bool registerParser(EXmlItem *(*createCallback)(EXmlModel*));
		void setValidationSchema(QUrl url);
		/**
			* @brief Called by save(QDomElement) to prepare model for saving.
			*
			* Reimplement this function, if anything needs to be done before
			* the model can be saved.
			*/
		virtual inline bool prepareForSaving(){ return true; }
		
		/**
			* append strings to this list, to get more columns
			*/
		QStringList headerList;

	protected slots:
		/**
			* @brief Reimplement this slot, if you want to recieve EXmlItem::changed() signals.
			*
			* This default implementation emits the dataChanged() signal for the complete
			* row represented by @p item.
			*/
		virtual void itemChanged(EXmlItem *item);
		/**
			* @brief Called by rootItem() signal EXmlItem::rowsAboutToBeInserted().
			*
			* It converts the recieved data to call beginInsertRows().
			*/
		virtual void itemRowsAboutToBeInserted(EXmlItem *parent, int start, int end);
		/**
			* @brief Called by rootItem() signal EXmlItem::rowsInserted().
			*
			* It calls endInsertRows().
			*/
		virtual void itemRowsInserted();
		/**
			* @brief Called by rootItem() signal EXmlItem::rowsAboutToBeRemoved().
			*
			* It calls beginRemoveRows() with converted recieved data.
			*/
		virtual void itemRowsAboutToBeRemoved(EXmlItem *parent, int start, int end);
		/**
			* @brief Called by rootItem() signal EXmlItem::rowsRemoved().
			*
			* It calls endRemoveRows().
			*/
		virtual void itemRowsRemoved();

	private slots:
		void setModified(bool status);
		
	private:
		QModelIndex indexFromItem(EXmlItem *find, EXmlItem *parent) const;
		
		QUrl validationUrl;
		QMap<QString, EXmlParserItem*> parsers;
		EXmlItem* myRootItem;
		bool _isModified;
};

#endif // EXMLMODEL_H
